/*
    This file is part of the 'yarec' package.
    Copyright (C) 1998  Ralf W. Stephan <stephan@tmt.de>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef UI_GTK

#include <gdk/gdk.h>
#include <gtk/gtk.h>
#include <gtk/gtkwidget.h>
#include "file/file.h"
#include "tasks/tasks.h"
#include "util/util.h"

extern char **play_xpm, **rec_xpm, **halt_xpm, **save_xpm, **stop_xpm, **quit_xpm;

static void destr_e(), sigb_play(), sigb_rec(), sigb_halt();
static void sigb_stop(), sigb_write();
static gint once(), in_task();

static GtkWidget* make_pbar();
static GtkWidget* make_buttons();
static GtkWidget* make_tbox();
static GtkStyle* make_style (gchar*, GdkColor, GdkColor);

static GtkWidget *w, *wstatus, *wpbar, *wtbox;
static GtkWidget *wl_rate, *wl_bits, *wl_chan;
static GtkWidget *wl_time, *wl_size, *wl_form;
static GtkWidget *wb_play, *wb_rec, *wb_stop, *wb_halt, *wb_quit, *wb_write;
static GtkStyle *st_norm, *st_big, *st_small;
static GdkColor c_white = { 0, 0xffff, 0xffff, 0xffff };
static GdkColor c_black = { 0, 0, 0, 0 };
static GdkColor c_rbc   = { 0, 0xffff, 0xffff, 0 };
static int first;

/*----------------- INTERFACE -----------------*/

void GtkUI_init (int* argcp, char*** argvp)
{
  gtk_init (argcp, argvp);
}

int GtkUI_is_commandline()
{
  return 0;
}

void GtkUI_print_to_console (const char* s1, const char* s2)
{
  if (!s2) s2 = "";
  g_print ("%s %s\n", s1, s2);
}

void GtkUI_draw_box()
{
  GtkWidget *vbox, *bbox, *wt, *pbarbox;
  char* cp;
  int i;
  
  w = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_widget_realize (w);

  cp = (char*)strcatalloc ("", Config_filename());
  if (strlen (cp) > 35)
  	cp[35] = '\0';
  gtk_window_set_title (GTK_WINDOW(w), cp);
  gtk_container_border_width (GTK_CONTAINER(w), 2);

  st_norm = gtk_widget_get_style (w);
  st_big = make_style ("-adobe-helvetica-medium-r-normal--*-240-*-*-*-*-*-*",
                       c_black, c_black);
  st_small = make_style ("-misc-fixed-medium-r-normal--10-100-75-75-c-60-iso8859-1",
                       c_black, c_black);
  
  vbox = gtk_vbox_new (FALSE, 0);
  wstatus = gtk_label_new ("");
  gtk_misc_set_alignment (GTK_MISC(wstatus), 0, 0.5);
  gtk_box_pack_start_defaults (GTK_BOX(vbox), wstatus);
  gtk_widget_show (wstatus);

  wtbox = make_tbox();
  gtk_box_pack_start_defaults (GTK_BOX(vbox), wtbox);
  gtk_widget_show (wtbox);

  pbarbox = make_pbar();
  gtk_box_pack_start_defaults (GTK_BOX(vbox), pbarbox);
  gtk_widget_show (pbarbox);
  
  bbox = make_buttons();
  gtk_box_pack_start_defaults (GTK_BOX(vbox), bbox);
  gtk_widget_show (bbox);
  
  gtk_container_add (GTK_CONTAINER(w), vbox);
  gtk_widget_show (vbox);
  gtk_widget_show (w);
}

void GtkUI_takes_over()
{
  gtk_signal_connect (GTK_OBJECT(w), "destroy", 
                      GTK_SIGNAL_FUNC(destr_e), NULL);

  gtk_signal_connect (GTK_OBJECT(wb_play), "clicked",
                      GTK_SIGNAL_FUNC(sigb_play), NULL);
  gtk_signal_connect (GTK_OBJECT(wb_rec), "clicked",
                      GTK_SIGNAL_FUNC(sigb_rec), NULL);
  gtk_signal_connect (GTK_OBJECT(wb_halt), "clicked",
                      GTK_SIGNAL_FUNC(sigb_halt), NULL);
  gtk_signal_connect (GTK_OBJECT(wb_stop), "clicked",
                      GTK_SIGNAL_FUNC(sigb_stop), NULL);
  gtk_signal_connect (GTK_OBJECT(wb_write), "clicked",
                      GTK_SIGNAL_FUNC(sigb_write), NULL);
  gtk_signal_connect (GTK_OBJECT(wb_quit), "clicked",
                      GTK_SIGNAL_FUNC(destr_e), NULL);

  sigpipe_handler (destr_e);  // Alt-F10 in fvwm2 causes it
  sigterm_handler (destr_e);  
  sigint_handler (destr_e);  

  first = 1;
  gtk_idle_add (once, 0);
  gtk_timeout_add (100, in_task, 0);   /* call timeout every 100 ms */
  gtk_main();
  Exception_normal ("Done GTK");  /* just to be sure */
}

void GtkUI_msg (const char* s)
{
  gtk_label_set (GTK_LABEL(wstatus), s);
  gtk_widget_show (wstatus);
}

void GtkUI_time (const char* s)
{
  gtk_label_set (GTK_LABEL(wl_time), s);
  gtk_widget_show (wl_time);
}

void GtkUI_size (long l1, long l2)
{
  char buf[16], tmp[8];
  float f;

  if (l1==0 || l2==0)
  	f=0.0;
  else
  	f = (float)l1/(float)l2;

  if (f<0.0) f = 0.0;
  if (f>1.0) f = 1.0;

  strcpy (tmp, l2si (l1));
  sprintf (buf, "%6s / %6s", tmp, l2si (l2));
  gtk_label_set (GTK_LABEL(wl_size), buf);
  gtk_widget_show (wl_size);
  
  gtk_progress_bar_update (GTK_PROGRESS_BAR(wpbar), f);
  gtk_widget_show (wpbar);
}

void GtkUI_rbc (int r, int b, int c)
{
  char buf[16];

  sprintf (buf, "%d Hz", r);
  gtk_label_set (GTK_LABEL(wl_rate), buf);
  gtk_widget_show (wl_rate);
  
  sprintf (buf, "%d bits", b);
  gtk_label_set (GTK_LABEL(wl_bits), buf);
  gtk_widget_show (wl_bits);

  gtk_label_set (GTK_LABEL(wl_chan), c<2?"mono":"stereo");
  gtk_widget_show (wl_chan);
}

void GtkUI_close()
{
  gtk_exit(0);
}

/*-------------- BUTTON MACHINE ------------------*/

static void sigb_play() 
{ 
  Task_schedule ('P');
}

static void sigb_rec()  
{ 
  Task_schedule ('R');
}

static void sigb_halt() 
{ 
  Task_schedule ('H');
}

static void sigb_stop() 
{ 
  Task_schedule ('S');
}

static void sigb_write()
{ 
  Task_schedule ('W');
}

static void destr_e() 
{  
  Task_schedule ('S');
  Exception_normal ("Done GTK");  
}

static gint once() 
{
  if (!first)
  	Exception_fatal ("Once() lies!");
  
  first = 0;
  if (Config_iget ("FORCE_ACTION")) 
  	gtk_signal_emit_by_name (GTK_OBJECT (Config_iget("RECORDING")? 
  	                                     wb_rec:wb_play),
  	                         "clicked");  
  return FALSE;
}

static gint in_task()
{
  int ret = Task_timeout();
  return TRUE;
}

/*--------------------- HELPERS -----------------------*/

static GtkWidget* make_icon (char**);

static GtkWidget* make_pbar()
{
  GtkWidget *ww, *b;

  b = gtk_vbox_new (FALSE, 0);
  gtk_widget_set_usize (b, 0, 13);
  ww = wpbar = gtk_progress_bar_new();
  gtk_widget_set_usize (ww, 0, 7);
  gtk_box_pack_start (GTK_BOX(b), ww, TRUE, TRUE, 3);
  gtk_widget_show (ww);
  return b;
}

static GtkWidget* make_buttons()
{
  GtkWidget *l, *bbox = gtk_hbox_new (FALSE, 0);

  wb_play = gtk_button_new();
  gtk_container_add (GTK_CONTAINER(wb_play), make_icon (play_xpm));
  gtk_box_pack_start_defaults (GTK_BOX(bbox), wb_play);
  gtk_widget_show (wb_play);

  wb_rec = gtk_button_new();
  gtk_container_add (GTK_CONTAINER(wb_rec), make_icon (rec_xpm));
  gtk_box_pack_start_defaults (GTK_BOX(bbox), wb_rec);
  gtk_widget_show (wb_rec);

  wb_halt = gtk_button_new();
  gtk_container_add (GTK_CONTAINER(wb_halt), make_icon (halt_xpm));
  gtk_box_pack_start_defaults (GTK_BOX(bbox), wb_halt);
  gtk_widget_show (wb_halt);

  wb_stop = gtk_button_new();
  gtk_container_add (GTK_CONTAINER(wb_stop), make_icon (stop_xpm));
  gtk_box_pack_start_defaults (GTK_BOX(bbox), wb_stop);
  gtk_widget_show (wb_stop);

  wb_write = gtk_button_new();
  gtk_container_add (GTK_CONTAINER(wb_write), make_icon (save_xpm));
  gtk_box_pack_start_defaults (GTK_BOX(bbox), wb_write);
  gtk_widget_show (wb_write);

  wb_quit = gtk_button_new();
  gtk_container_add (GTK_CONTAINER(wb_quit), make_icon (quit_xpm));
  gtk_box_pack_start_defaults (GTK_BOX(bbox), wb_quit);
  gtk_widget_show (wb_quit);

  return bbox;
}

static GtkWidget* make_tbox()
{
  GtkWidget *box, *row1, *row2, *l;
  
  box = gtk_vbox_new (FALSE, 0);

  row1 = gtk_hbox_new (FALSE, 0);
  wl_rate = l = gtk_label_new ("");
  gtk_misc_set_alignment (GTK_MISC(l), 1, 0.5);
  gtk_widget_set_style (l, st_small);
  gtk_box_pack_start_defaults (GTK_BOX(row1), l);
  gtk_widget_show (l);
  wl_bits = l = gtk_label_new ("");
  gtk_misc_set_alignment (GTK_MISC(l), 1, 0.5);
  gtk_widget_set_style (l, st_small);
  gtk_box_pack_start_defaults (GTK_BOX(row1), l);
  gtk_widget_show (l);
  wl_chan = l = gtk_label_new ("");
  gtk_widget_set_style (l, st_small);
  gtk_box_pack_start_defaults (GTK_BOX(row1), l);
  gtk_widget_show (l);
  gtk_box_pack_start_defaults (GTK_BOX(box), row1);
  gtk_widget_show (row1);

  row2 = gtk_hbox_new (FALSE, 0);
  wl_time = l = gtk_label_new ("00:00:00");
  gtk_misc_set_alignment (GTK_MISC(l), 0, 0.5);
  gtk_widget_set_style (l, st_big);
  gtk_box_pack_start_defaults (GTK_BOX(row2), l);
  gtk_widget_show (l);
  wl_size = l = gtk_label_new ("0 / 0");
  gtk_misc_set_alignment (GTK_MISC(l), 1, 0.5);
  gtk_box_pack_start_defaults (GTK_BOX(row2), l);
  gtk_widget_show (l);
  gtk_box_pack_start_defaults (GTK_BOX(box), row2);
  gtk_widget_show (row2);

  return box;
}

static GtkWidget* make_icon (char** xc)
{
  GtkWidget *pw, *box;
  GdkPixmap *pm;
  GtkStyle *style;
  GdkBitmap *mask = 0;
  
  style = gtk_widget_get_style (w);
  pm = gdk_pixmap_create_from_xpm_d (w->window, &mask, 
                                   &style->bg[GTK_STATE_NORMAL], (gchar**)xc);
  pw = gtk_pixmap_new (pm, mask);
  gtk_widget_show (pw);
  
  return pw;
}

static GtkStyle* make_style (gchar* fn, GdkColor c1, GdkColor c2)
{
  GtkStyle *style;
  GdkFont *font;
  
  style = gtk_style_new();
  font = gdk_font_load (fn);
  style->font = font;
  gdk_font_ref (font);
  style->fg[GTK_STATE_NORMAL] = c1;
  style->bg[GTK_STATE_NORMAL] = c2;
  return style;
}

#endif
